@@ -32,10 +32,14 @@
 #include "genericfader.h"
 #include "fadechannel.h"
 #include "rgbmatrix.h"
+#include "rgbimage.h"
 #include "doc.h"
 
 #define KXMLQLCRGBMatrixStartColor      QString("MonoColor")
 #define KXMLQLCRGBMatrixEndColor        QString("EndColor")
+#define KXMLQLCRGBMatrixColor           QString("Color")
+#define KXMLQLCRGBMatrixColorIndex      QString("Index")
+
 #define KXMLQLCRGBMatrixFixtureGroup    QString("FixtureGroup")
 #define KXMLQLCRGBMatrixDimmerControl   QString("DimmerControl")
 
@@ -64,8 +68,6 @@ RGBMatrix::RGBMatrix(Doc* doc)
 #if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
     , m_algorithmMutex(QMutex::Recursive)
 #endif
-    , m_startColor(Qt::red)
-    , m_endColor(QColor())
     , m_stepHandler(new RGBMatrixStep())
     , m_roundTime(new QElapsedTimer())
     , m_stepsCount(0)
@@ -75,6 +77,9 @@ RGBMatrix::RGBMatrix(Doc* doc)
     setName(tr("New RGB Matrix"));
     setDuration(500);
 
+    m_rgbColors.fill(QColor(), RGBAlgorithmColorDisplayCount);
+    setColor(0, Qt::red);
+
     RGBScript scr = doc->rgbScriptsCache()->script("Stripes");
     setAlgorithm(scr.clone());
 }
@@ -117,7 +122,7 @@ quint32 RGBMatrix::totalDuration()
     if (grp == NULL)
         return 0;
 
-    qDebug () << "Algorithm steps:" << m_algorithm->rgbMapStepCount(grp->size());
+    //qDebug () << "Algorithm steps:" << m_algorithm->rgbMapStepCount(grp->size());
     return m_algorithm->rgbMapStepCount(grp->size()) * duration();
 }
 
@@ -162,12 +167,16 @@ bool RGBMatrix::copyFrom(const Function* function)
 
     setDimmerControl(mtx->dimmerControl());
     setFixtureGroup(mtx->fixtureGroup());
+
+    m_rgbColors.clear();
+    foreach (QColor col, mtx->getColors())
+        m_rgbColors.append(col);
+
     if (mtx->algorithm() != NULL)
         setAlgorithm(mtx->algorithm()->clone());
     else
         setAlgorithm(NULL);
-    setStartColor(mtx->startColor());
-    setEndColor(mtx->endColor());
+
     setControlMode(mtx->controlMode());
 
     return Function::copyFrom(function);
@@ -277,54 +286,78 @@ void RGBMatrix::previewMap(int step, RGBMatrixStep *handler)
         m_group = doc()->fixtureGroup(fixtureGroup());
 
     if (m_group != NULL)
+    {
+        setMapColors();
         m_algorithm->rgbMap(m_group->size(), handler->stepColor().rgb(), step, handler->m_map);
+    }
 }
 
 /****************************************************************************
  * Color
  ****************************************************************************/
 
-void RGBMatrix::setStartColor(const QColor& c)
+void RGBMatrix::setColor(int i, QColor c)
 {
-    m_startColor = c;
+    if (i < 0)
+        return;
+
+    if (i >= m_rgbColors.count())
+        m_rgbColors.resize(i + 1);
+
+    m_rgbColors.replace(i, c);
     {
         QMutexLocker algorithmLocker(&m_algorithmMutex);
         if (m_algorithm != NULL)
         {
-            m_algorithm->setColors(m_startColor, m_endColor);
+            m_algorithm->setColors(m_rgbColors);
             updateColorDelta();
         }
     }
+    setMapColors();
     emit changed(id());
 }
 
-QColor RGBMatrix::startColor() const
+QColor RGBMatrix::getColor(int i) const
 {
-    return m_startColor;
+    if (i < 0 || i >= m_rgbColors.count())
+        return QColor();
+
+    return m_rgbColors.at(i);
 }
 
-void RGBMatrix::setEndColor(const QColor &c)
+QVector<QColor> RGBMatrix::getColors() const
 {
-    m_endColor = c;
-    {
-        QMutexLocker algorithmLocker(&m_algorithmMutex);
-        if (m_algorithm != NULL)
-        {
-            m_algorithm->setColors(m_startColor, m_endColor);
-            updateColorDelta();
-        }
-    }
-    emit changed(id());
-}
-
-QColor RGBMatrix::endColor() const
-{
-    return m_endColor;
+    return m_rgbColors;
 }
 
 void RGBMatrix::updateColorDelta()
 {
-    m_stepHandler->calculateColorDelta(m_startColor, m_endColor);
+    m_stepHandler->calculateColorDelta(m_rgbColors[0], m_rgbColors[1], m_algorithm);
+}
+
+void RGBMatrix::setMapColors()
+{
+    QMutexLocker algorithmLocker(&m_algorithmMutex);
+    if (m_algorithm == NULL)
+        return;
+
+    if (m_algorithm->apiVersion() < 3)
+        return;
+
+    if (m_group == NULL)
+        m_group = doc()->fixtureGroup(fixtureGroup());
+
+    if (m_group != NULL)
+    {
+        QVector<unsigned int> rawColors;
+        for (int i = 0; i < m_algorithm->acceptColors(); i++)
+        {
+            QColor col = m_rgbColors.at(i);
+            rawColors.append(col.isValid() ? col.rgb() : 0);
+        }
+
+        m_algorithm->rgbMapSetColors(rawColors);
+    }
 }
 
 /************************************************************************
@@ -339,6 +372,10 @@ void RGBMatrix::setProperty(QString propName, QString value)
     {
         RGBScript *script = static_cast<RGBScript*> (m_algorithm);
         script->setProperty(propName, value);
+
+        QVector<uint> colors = script->rgbMapGetColors();
+        for (int i = 0; i < colors.count(); i++)
+            setColor(i, QColor::fromRgb(colors.at(i)));
     }
     m_stepsCount = stepsCount();
 }
@@ -402,13 +439,19 @@ bool RGBMatrix::loadXML(QXmlStreamReader &root)
         {
             loadXMLRunOrder(root);
         }
+        // Legacy support
         else if (root.name() == KXMLQLCRGBMatrixStartColor)
         {
-            setStartColor(QColor::fromRgb(QRgb(root.readElementText().toUInt())));
+            setColor(0, QColor::fromRgb(QRgb(root.readElementText().toUInt())));
         }
         else if (root.name() == KXMLQLCRGBMatrixEndColor)
         {
-            setEndColor(QColor::fromRgb(QRgb(root.readElementText().toUInt())));
+            setColor(1, QColor::fromRgb(QRgb(root.readElementText().toUInt())));
+        }
+        else if (root.name() == KXMLQLCRGBMatrixColor)
+        {
+            int colorIdx = root.attributes().value(KXMLQLCRGBMatrixColorIndex).toInt();
+            setColor(colorIdx, QColor::fromRgb(QRgb(root.readElementText().toUInt())));
         }
         else if (root.name() == KXMLQLCRGBMatrixControlMode)
         {
@@ -462,12 +505,14 @@ bool RGBMatrix::saveXML(QXmlStreamWriter *doc)
     if (dimmerControl())
         doc->writeTextElement(KXMLQLCRGBMatrixDimmerControl, QString::number(dimmerControl()));
 
-    /* Start Color */
-    doc->writeTextElement(KXMLQLCRGBMatrixStartColor, QString::number(startColor().rgb()));
-
-    /* End Color */
-    if (endColor().isValid())
-        doc->writeTextElement(KXMLQLCRGBMatrixEndColor, QString::number(endColor().rgb()));
+    /* Colors */
+    for (int i = 0; i < m_rgbColors.count(); i++)
+    {
+        doc->writeStartElement(KXMLQLCRGBMatrixColor);
+        doc->writeAttribute(KXMLQLCRGBMatrixColorIndex, QString::number(i));
+        doc->writeCharacters(QString::number(m_rgbColors.at(i).rgb()));
+        doc->writeEndElement();
+    }
 
     /* Control Mode */
     doc->writeTextElement(KXMLQLCRGBMatrixControlMode, RGBMatrix::controlModeToString(m_controlMode));
@@ -526,7 +571,7 @@ void RGBMatrix::preRun(MasterTimer *timer)
         if (m_algorithm != NULL)
         {
             // Copy direction from parent class direction
-            m_stepHandler->initializeDirection(direction(), m_startColor, m_endColor, m_stepsCount);
+            m_stepHandler->initializeDirection(direction(), m_rgbColors[0], m_rgbColors[1], m_stepsCount, m_algorithm);
 
             if (m_algorithm->type() == RGBAlgorithm::Script)
             {
@@ -538,6 +583,12 @@ void RGBMatrix::preRun(MasterTimer *timer)
                     script->setProperty(it.key(), it.value());
                 }
             }
+            else if (m_algorithm->type() == RGBAlgorithm::Image)
+            {
+                RGBImage *image = static_cast<RGBImage*> (m_algorithm);
+                if (image->animatedSource())
+                    image->rewindAnimation();
+            }
         }
     }
 
@@ -664,7 +715,7 @@ void RGBMatrix::roundCheck()
     if (m_algorithm == NULL)
         return;
 
-    if (m_stepHandler->checkNextStep(runOrder(), m_startColor, m_endColor, m_stepsCount) == false)
+    if (m_stepHandler->checkNextStep(runOrder(), m_rgbColors[0], m_rgbColors[1], m_stepsCount) == false)
         stop(FunctionParent::master());
 
     m_roundTime->restart();
@@ -675,21 +726,24 @@ void RGBMatrix::roundCheck()
         roundElapsed(duration());
 }
 
-FadeChannel *RGBMatrix::getFader(QList<Universe *> universes, quint32 universeID, quint32 fixtureID, quint32 channel)
+FadeChannel *RGBMatrix::getFader(Universe *universe, quint32 fixtureID, quint32 channel)
 {
     // get the universe Fader first. If doesn't exist, create it
-    QSharedPointer<GenericFader> fader = m_fadersMap.value(universeID, QSharedPointer<GenericFader>());
+    if (universe == NULL)
+        return NULL;
+
+    QSharedPointer<GenericFader> fader = m_fadersMap.value(universe->id(), QSharedPointer<GenericFader>());
     if (fader.isNull())
     {
-        fader = universes[universeID]->requestFader();
+        fader = universe->requestFader();
         fader->adjustIntensity(getAttributeValue(Intensity));
         fader->setBlendMode(blendMode());
         fader->setName(name());
         fader->setParentFunctionID(id());
-        m_fadersMap[universeID] = fader;
+        m_fadersMap[universe->id()] = fader;
     }
 
-    return fader->getChannelFader(doc(), universes[universeID], fixtureID, channel);
+    return fader->getChannelFader(doc(), universe, fixtureID, channel);
 }
 
 void RGBMatrix::updateFaderValues(FadeChannel *fc, uchar value, uint fadeTime)
@@ -726,86 +780,46 @@ void RGBMatrix::updateMapChannels(const RGBMap& map, const FixtureGroup *grp, QL
             continue;
 
         uint col = map[pt.y()][pt.x()];
+        QVector<quint32> channelList;
+        QVector<uchar> valueList;
 
         if (m_controlMode == ControlModeRgb)
         {
-            QVector <quint32> rgb = head.rgbChannels();
-            QVector <quint32> cmy = head.cmyChannels();
+            channelList = head.rgbChannels();
 
-            if (rgb.size() == 3)
+            if (channelList.size() == 3)
             {
-                // RGB color mixing
-                FadeChannel *fc = getFader(universes, fxi->universe(), grpHead.fxi, rgb.at(0));
-                updateFaderValues(fc, qRed(col), fadeTime);
-
-                fc = getFader(universes, fxi->universe(), grpHead.fxi, rgb.at(1));
-                updateFaderValues(fc, qGreen(col), fadeTime);
-
-                fc = getFader(universes, fxi->universe(), grpHead.fxi, rgb.at(2));
-                updateFaderValues(fc, qBlue(col), fadeTime);
+                valueList.append(qRed(col));
+                valueList.append(qGreen(col));
+                valueList.append(qBlue(col));
             }
-            else if (cmy.size() == 3)
+            else
             {
-                // CMY color mixing
-                QColor cmyCol(col);
+                channelList = head.cmyChannels();
 
-                FadeChannel *fc = getFader(universes, fxi->universe(), grpHead.fxi, cmy.at(0));
-                updateFaderValues(fc, cmyCol.cyan(), fadeTime);
-
-                fc = getFader(universes, fxi->universe(), grpHead.fxi, cmy.at(1));
-                updateFaderValues(fc, cmyCol.magenta(), fadeTime);
-
-                fc = getFader(universes, fxi->universe(), grpHead.fxi, cmy.at(2));
-                updateFaderValues(fc, cmyCol.yellow(), fadeTime);
-            }
-        }
-        else if (m_controlMode == ControlModeWhite)
-        {
-            quint32 white = head.channelNumber(QLCChannel::White, QLCChannel::MSB);
-
-            if (white != QLCChannel::invalid())
-            {
-                FadeChannel *fc = getFader(universes, fxi->universe(), grpHead.fxi, white);
-                updateFaderValues(fc, rgbToGrey(col), fadeTime);
-            }
-        }
-        else if (m_controlMode == ControlModeAmber)
-        {
-            quint32 amber = head.channelNumber(QLCChannel::Amber, QLCChannel::MSB);
-
-            if (amber != QLCChannel::invalid())
-            {
-                FadeChannel *fc = getFader(universes, fxi->universe(), grpHead.fxi, amber);
-                updateFaderValues(fc, rgbToGrey(col), fadeTime);
-            }
-        }
-        else if (m_controlMode == ControlModeUV)
-        {
-            quint32 uv = head.channelNumber(QLCChannel::UV, QLCChannel::MSB);
-
-            if (uv != QLCChannel::invalid())
-            {
-                FadeChannel *fc = getFader(universes, fxi->universe(), grpHead.fxi, uv);
-                updateFaderValues(fc, rgbToGrey(col), fadeTime);
+                if (channelList.size() == 3)
+                {
+                    // CMY color mixing
+                    QColor cmyCol(col);
+                    valueList.append(cmyCol.cyan());
+                    valueList.append(cmyCol.magenta());
+                    valueList.append(cmyCol.yellow());
+                }
             }
         }
         else if (m_controlMode == ControlModeShutter)
         {
-            QVector <quint32> shutters = head.shutterChannels();
+            channelList = head.shutterChannels();
 
-            if (shutters.size())
+            if (channelList.size())
             {
-                FadeChannel *fc = getFader(universes, fxi->universe(), grpHead.fxi, shutters.first());
-                updateFaderValues(fc, rgbToGrey(col), fadeTime);
+                // make sure only one channel is in the list
+                channelList.resize(1);
+                valueList.append(rgbToGrey(col));
             }
         }
-
-        if (m_controlMode == ControlModeDimmer || m_dimmerControl)
+        else if (m_controlMode == ControlModeDimmer || m_dimmerControl)
         {
-            quint32 masterDim = fxi->masterIntensityChannel();
-            quint32 headDim = head.channelNumber(QLCChannel::Intensity, QLCChannel::MSB);
-            QVector <quint32> dimmers;
-
             // Collect all dimmers that affect current head:
             // They are the master dimmer (affects whole fixture)
             // and per-head dimmer.
@@ -823,26 +837,44 @@ void RGBMatrix::updateMapChannels(const RGBMap& map, const FixtureGroup *grp, QL
             // *least important - per head dimmer if present,
             // otherwise per fixture dimmer if present
 
+            quint32 masterDim = fxi->masterIntensityChannel();
+            quint32 headDim = head.channelNumber(QLCChannel::Intensity, QLCChannel::MSB);
+
             if (masterDim != QLCChannel::invalid())
-                dimmers << masterDim;
+            {
+                channelList.append(masterDim);
+                valueList.append(rgbToGrey(col));
+            }
 
             if (headDim != QLCChannel::invalid() && headDim != masterDim)
-                dimmers << headDim;
-
-            if (dimmers.size())
             {
-                // Set dimmer to value of the color (e.g. for PARs)
-                FadeChannel *fc = getFader(universes, fxi->universe(), grpHead.fxi, dimmers.last());
-                updateFaderValues(fc, rgbToGrey(col), fadeTime);
-                dimmers.pop_back();
+                channelList.append(headDim);
+                valueList.append(col == 0 ? 0 : 255);
             }
+        }
+        else
+        {
+            if (m_controlMode == ControlModeWhite)
+                channelList.append(head.channelNumber(QLCChannel::White, QLCChannel::MSB));
+            else if (m_controlMode == ControlModeAmber)
+                channelList.append(head.channelNumber(QLCChannel::Amber, QLCChannel::MSB));
+            else if (m_controlMode == ControlModeUV)
+                channelList.append(head.channelNumber(QLCChannel::UV, QLCChannel::MSB));
 
-            // Set the rest of the dimmer channels to full on
-            foreach (quint32 ch, dimmers)
-            {
-                FadeChannel *fc = getFader(universes, fxi->universe(), grpHead.fxi, ch);
-                updateFaderValues(fc, col == 0 ? 0 : 255, fadeTime);
-            }
+            valueList.append(rgbToGrey(col));
+        }
+
+        quint32 absAddress = fxi->universeAddress();
+
+        for (int i = 0; i < channelList.count(); i++)
+        {
+            if (channelList.at(i) == QLCChannel::invalid())
+                continue;
+
+            quint32 universeIndex = floor((absAddress + channelList.at(i)) / 512);
+
+            FadeChannel *fc = getFader(universes.at(universeIndex), grpHead.fxi, channelList.at(i));
+            updateFaderValues(fc, valueList.at(i), fadeTime);
         }
     }
 }
@@ -980,13 +1012,13 @@ int RGBMatrixStep::currentStepIndex() const
     return m_currentStepIndex;
 }
 
-void RGBMatrixStep::calculateColorDelta(QColor startColor, QColor endColor)
+void RGBMatrixStep::calculateColorDelta(QColor startColor, QColor endColor, RGBAlgorithm *algorithm)
 {
     m_crDelta = 0;
     m_cgDelta = 0;
     m_cbDelta = 0;
 
-    if (endColor.isValid())
+    if (endColor.isValid() && algorithm != NULL && algorithm->acceptColors() > 1)
     {
         m_crDelta = endColor.red() - startColor.red();
         m_cgDelta = endColor.green() - startColor.green();
@@ -1025,7 +1057,7 @@ void RGBMatrixStep::updateStepColor(int stepIndex, QColor startColor, int stepsC
     //qDebug() << "RGBMatrix step" << stepIndex << ", color:" << QString::number(m_stepColor.rgb(), 16);
 }
 
-void RGBMatrixStep::initializeDirection(Function::Direction direction, QColor startColor, QColor endColor, int stepsCount)
+void RGBMatrixStep::initializeDirection(Function::Direction direction, QColor startColor, QColor endColor, int stepsCount, RGBAlgorithm *algorithm)
 {
     m_direction = direction;
 
@@ -1044,7 +1076,7 @@ void RGBMatrixStep::initializeDirection(Function::Direction direction, QColor st
             setStepColor(startColor);
     }
 
-    calculateColorDelta(startColor, endColor);
+    calculateColorDelta(startColor, endColor, algorithm);
 }
 
 bool RGBMatrixStep::checkNextStep(Function::RunOrder order,
