@@ -91,6 +91,12 @@ public:
     /** @reimp */
     int rgbMapStepCount(const QSize& size);
 
+    /** @reimp */
+    void rgbMapSetColors(QVector<uint> &colors);
+
+    /** @reimp */
+    QVector<uint> rgbMapGetColors();
+
     /** @reimp */
     void rgbMap(const QSize& size, uint rgb, int step, RGBMap &map);
 
@@ -120,6 +126,8 @@ private:
     QJSValue m_script;          //! The script itself
     QJSValue m_rgbMap;          //! rgbMap() function
     QJSValue m_rgbMapStepCount; //! rgbMapStepCount() function
+    QJSValue m_rgbMapSetColors; //! rgbMapSetColors() function
+    QJSValue m_rgbMapGetColors; //! rgbMapSetColors() function
 
     /************************************************************************
      * Properties
